import { useRef, useEffect } from 'react';
import '../utils/useId.js';
import '../utils/useCreateElement.js';
import '../system/SystemContext.js';
import '../chunk-64f351ca.js';
import { mergeProps } from '../utils/mergeProps.js';
import { a as _objectWithoutPropertiesLoose, b as _objectSpread } from '../chunk-397c8002.js';
import { unstable_createComponent } from '../utils/createComponent.js';
import '../system/useToken.js';
import { unstable_useProps } from '../system/useProps.js';
import { unstable_useOptions } from '../system/useOptions.js';
import '../Box/Box.js';
import '../chunk-6e18ef4b.js';
import '../Tabbable/Tabbable.js';
import '../Button/Button.js';
import '../chunk-459cc222.js';
import '../chunk-e555b6ea.js';
import '../Hidden/HiddenState.js';
import '../Dialog/DialogState.js';
import '../Hidden/HiddenDisclosure.js';
import '../Dialog/DialogDisclosure.js';
import '../chunk-beb7e682.js';
import '../Rover/RoverState.js';
import '../Rover/Rover.js';
import 'popper.js';
import '../Popover/PopoverState.js';
import { useMenuState } from './MenuState.js';
import '../Popover/PopoverDisclosure.js';
import { useMenuDisclosure } from './MenuDisclosure.js';
import { useMenuItem } from './MenuItem.js';

function useKeyboardFocus(targetRef, handler, enabled) {
  var handlerRef = useRef(handler);
  var lastMouseDown = useRef();
  useEffect(function () {
    handlerRef.current = handler;
  });
  useEffect(function () {
    if (!enabled || !targetRef.current) return undefined;

    var handleFocus = function handleFocus(event) {
      if (event.target !== lastMouseDown.current) {
        handlerRef.current(event);
      }
    };

    var handleBlur = function handleBlur() {
      lastMouseDown.current = null;
    };

    var handleMouseDown = function handleMouseDown(event) {
      lastMouseDown.current = event.target;
    };

    targetRef.current.addEventListener("focus", handleFocus);
    targetRef.current.addEventListener("blur", handleBlur);
    document.addEventListener("mousedown", handleMouseDown);
    document.addEventListener("touchstart", handleMouseDown);
    return function () {
      if (targetRef.current) {
        targetRef.current.removeEventListener("focus", handleFocus);
        targetRef.current.removeEventListener("blur", handleBlur);
      }

      document.removeEventListener("mousedown", handleMouseDown);
      document.removeEventListener("touchstart", handleMouseDown);
    };
  }, []);
}

function unstable_useMenuItemDisclosure(_ref, htmlProps) {
  if (htmlProps === void 0) {
    htmlProps = {};
  }

  var _ref$unstable_focusab = _ref.unstable_focusable,
      unstable_focusable = _ref$unstable_focusab === void 0 ? true : _ref$unstable_focusab,
      options = _objectWithoutPropertiesLoose(_ref, ["unstable_focusable"]);

  var _options = _objectSpread({
    unstable_focusable: unstable_focusable
  }, options);

  _options = unstable_useOptions("useMenuItemDisclosure", _options, htmlProps);
  var ref = useRef(null);
  var _options2 = _options,
      parent = _options2.unstable_parent;

  if (!parent) {
    // TODO: Better error
    throw new Error("Missing parent prop");
  }

  useKeyboardFocus(ref, _options.show, parent.orientation === "horizontal" && !_options.disabled);
  useEffect(function () {
    if (parent.orientation !== "horizontal") return;
    var thisStop = parent.unstable_stops.find(function (stop) {
      return stop.ref.current === ref.current;
    });
    var thisIsCurrent = thisStop && thisStop.id === parent.unstable_currentId;

    if (!thisIsCurrent && _options.visible) {
      _options.hide();
    }
  }, [parent.orientation, parent.unstable_currentId, parent.unstable_stops, _options.hide]);
  htmlProps = mergeProps({
    ref: ref,
    onKeyDown: function onKeyDown(event) {
      if (event.key === "Escape") {
        event.preventDefault();

        _options.hide();
      }
    }
  }, htmlProps);
  htmlProps = useMenuItem(_objectSpread({}, _options, parent), htmlProps);
  htmlProps = useMenuDisclosure(_options, htmlProps);
  htmlProps = unstable_useProps("useMenuItemDisclosure", _options, htmlProps);
  return htmlProps;
}
var keys = [].concat(useMenuDisclosure.__keys, useMenuState.__keys, ["stopId"]);
unstable_useMenuItemDisclosure.__keys = keys;
var unstable_MenuItemDisclosure = unstable_createComponent({
  as: "button",
  useHook: unstable_useMenuItemDisclosure
});

export { unstable_MenuItemDisclosure, unstable_useMenuItemDisclosure };
