import { useState, useEffect } from 'react';
import { unstable_useCreateElement } from '../utils/useCreateElement.js';
import '../system/SystemContext.js';
import '../chunk-64f351ca.js';
import { mergeProps } from '../utils/mergeProps.js';
import '../chunk-397c8002.js';
import { unstable_createComponent } from '../utils/createComponent.js';
import '../system/useToken.js';
import { unstable_useProps } from '../system/useProps.js';
import { unstable_useOptions } from '../system/useOptions.js';
import { useBox } from '../Box/Box.js';
import '../chunk-459cc222.js';
import { a as warning } from '../chunk-e555b6ea.js';
import '../Rover/RoverState.js';
import { unstable_useStaticMenuState } from './StaticMenuState.js';

function useShortcuts(_ref, timeout) {
  var stops = _ref.unstable_stops,
      move = _ref.unstable_move;

  if (timeout === void 0) {
    timeout = 500;
  }

  var _React$useState = useState(""),
      keys = _React$useState[0],
      setKeys = _React$useState[1];

  useEffect(function () {
    if (!keys) return undefined;
    var timeoutId = setTimeout(function () {
      return setKeys("");
    }, timeout);
    var stop = stops.find(function (s) {
      return Boolean(s.ref.current && s.ref.current.textContent && s.ref.current.textContent.toLowerCase().startsWith(keys));
    });

    if (stop) {
      move(stop.id);
    }

    return function () {
      return clearTimeout(timeoutId);
    };
  }, [keys, stops, move]);
  return function (event) {
    if (event.metaKey || event.altKey || event.shiftKey || event.ctrlKey) {
      return;
    }

    if (/^[a-z0-9_-]$/i.test(event.key)) {
      event.preventDefault();
      setKeys("" + keys + event.key);
    }
  };
}

function unstable_useStaticMenu(options, htmlProps) {
  if (htmlProps === void 0) {
    htmlProps = {};
  }

  options = unstable_useOptions("useStaticMenu", options, htmlProps);
  var onKeyDown = useShortcuts(options);
  var ariaOwns = options.unstable_stops.map(function (stop) {
    var ariaControls = stop.ref.current.getAttribute("aria-controls");
    if (ariaControls) return stop.id + " " + ariaControls;
    return stop.id;
  }).join(" ");
  htmlProps = mergeProps({
    role: options.orientation === "horizontal" ? "menubar" : "menu",
    "aria-orientation": options.orientation,
    "aria-owns": ariaOwns,
    onKeyDown: onKeyDown
  }, htmlProps);
  htmlProps = useBox(options, htmlProps);
  htmlProps = unstable_useProps("useStaticMenu", options, htmlProps);
  return htmlProps;
}
var keys = [].concat(useBox.__keys, unstable_useStaticMenuState.__keys);
unstable_useStaticMenu.__keys = keys;
var unstable_StaticMenu = unstable_createComponent({
  as: "div",
  useHook: unstable_useStaticMenu,
  useCreateElement: function useCreateElement(type, props, children) {
    process.env.NODE_ENV !== "production" ? warning(!props["aria-label"] && !props["aria-labelledby"] && props.role !== "menubar", "You should provide either `aria-label` or `aria-labelledby` props.\nSee https://www.w3.org/TR/wai-aria-practices-1.1/#wai-aria-roles-states-and-properties-13", "StaticMenu") : void 0;
    return unstable_useCreateElement(type, props, children);
  }
});

export { unstable_StaticMenu, unstable_useStaticMenu };
