'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var React = require('react');
var useCreateElement = require('../utils/useCreateElement.js');
require('../system/SystemContext.js');
require('../chunk-6fc382c5.js');
var mergeProps = require('../utils/mergeProps.js');
require('../chunk-37dea005.js');
var createComponent = require('../utils/createComponent.js');
require('../system/useToken.js');
var useProps = require('../system/useProps.js');
var useOptions = require('../system/useOptions.js');
var Box = require('../Box/Box.js');
require('../chunk-eb7a57a2.js');
var __chunk_6 = require('../chunk-ebc3ed80.js');
require('../Rover/RoverState.js');
var StaticMenuState = require('./StaticMenuState.js');

function useShortcuts(_ref, timeout) {
  var stops = _ref.unstable_stops,
      move = _ref.unstable_move;

  if (timeout === void 0) {
    timeout = 500;
  }

  var _React$useState = React.useState(""),
      keys = _React$useState[0],
      setKeys = _React$useState[1];

  React.useEffect(function () {
    if (!keys) return undefined;
    var timeoutId = setTimeout(function () {
      return setKeys("");
    }, timeout);
    var stop = stops.find(function (s) {
      return Boolean(s.ref.current && s.ref.current.textContent && s.ref.current.textContent.toLowerCase().startsWith(keys));
    });

    if (stop) {
      move(stop.id);
    }

    return function () {
      return clearTimeout(timeoutId);
    };
  }, [keys, stops, move]);
  return function (event) {
    if (event.metaKey || event.altKey || event.shiftKey || event.ctrlKey) {
      return;
    }

    if (/^[a-z0-9_-]$/i.test(event.key)) {
      event.preventDefault();
      setKeys("" + keys + event.key);
    }
  };
}

function unstable_useStaticMenu(options, htmlProps) {
  if (htmlProps === void 0) {
    htmlProps = {};
  }

  options = useOptions.unstable_useOptions("useStaticMenu", options, htmlProps);
  var onKeyDown = useShortcuts(options);
  var ariaOwns = options.unstable_stops.map(function (stop) {
    var ariaControls = stop.ref.current.getAttribute("aria-controls");
    if (ariaControls) return stop.id + " " + ariaControls;
    return stop.id;
  }).join(" ");
  htmlProps = mergeProps.mergeProps({
    role: options.orientation === "horizontal" ? "menubar" : "menu",
    "aria-orientation": options.orientation,
    "aria-owns": ariaOwns,
    onKeyDown: onKeyDown
  }, htmlProps);
  htmlProps = Box.useBox(options, htmlProps);
  htmlProps = useProps.unstable_useProps("useStaticMenu", options, htmlProps);
  return htmlProps;
}
var keys = [].concat(Box.useBox.__keys, StaticMenuState.unstable_useStaticMenuState.__keys);
unstable_useStaticMenu.__keys = keys;
var unstable_StaticMenu = createComponent.unstable_createComponent({
  as: "div",
  useHook: unstable_useStaticMenu,
  useCreateElement: function useCreateElement$1(type, props, children) {
    process.env.NODE_ENV !== "production" ? __chunk_6.warning(!props["aria-label"] && !props["aria-labelledby"] && props.role !== "menubar", "You should provide either `aria-label` or `aria-labelledby` props.\nSee https://www.w3.org/TR/wai-aria-practices-1.1/#wai-aria-roles-states-and-properties-13", "StaticMenu") : void 0;
    return useCreateElement.unstable_useCreateElement(type, props, children);
  }
});

exports.unstable_StaticMenu = unstable_StaticMenu;
exports.unstable_useStaticMenu = unstable_useStaticMenu;
