import * as React from "react";
import { unstable_HiddenOptions, unstable_HiddenProps } from "../Hidden/Hidden";
import { unstable_DialogStateReturn } from "./DialogState";
export declare type unstable_DialogOptions = unstable_HiddenOptions & Partial<unstable_DialogStateReturn> & Pick<unstable_DialogStateReturn, "unstable_hiddenId"> & {
    /**
     * Toggles Dialog's `modal` state.
     *  - Non-modal: `preventBodyScroll` doesn't work and focus is free.
     *  - Modal: `preventBodyScroll` is automatically enabled and focus is
     * trapped within the dialog.
     * @default true
     */
    unstable_modal?: boolean;
    /**
     * When enabled, user can hide the dialog by pressing `Escape`.
     * @default true
     */
    unstable_hideOnEsc?: boolean;
    /**
     * When enabled, user can hide the dialog by clicking outside it.
     * @default true
     */
    unstable_hideOnClickOutside?: boolean;
    /**
     * When enabled, user can't scroll on body when the dialog is visible.
     * This option doesn't work if the dialog isn't modal.
     * @default true
     */
    unstable_preventBodyScroll?: boolean;
    /**
     * The element that will be focused when the dialog shows.
     * When not set, the first tabbable element within the dialog will be used.
     * `autoFocusOnShow` disables it.
     */
    unstable_initialFocusRef?: React.RefObject<HTMLElement>;
    /**
     * The element that will be focused when the dialog hides.
     * When not set, the disclosure component will be used.
     * `autoFocusOnHide` disables it.
     */
    unstable_finalFocusRef?: React.RefObject<HTMLElement>;
    /**
     * Whether or not to move focus when the dialog shows.
     * @default true
     */
    unstable_autoFocusOnShow?: boolean;
    /**
     * Whether or not to move focus when the dialog hides.
     * @default true
     */
    unstable_autoFocusOnHide?: boolean;
};
export declare type unstable_DialogProps = unstable_HiddenProps;
export declare function useDialog({ unstable_modal, unstable_hideOnEsc, unstable_hideOnClickOutside, unstable_preventBodyScroll, unstable_autoFocusOnShow, unstable_autoFocusOnHide, ...options }: unstable_DialogOptions, htmlProps?: unstable_DialogProps): React.HTMLAttributes<any> & React.RefAttributes<any>;
export declare namespace useDialog {
    var __keys: ("unstable_system" | "show" | "hide" | "unstable_hiddenId" | "visible" | "toggle" | "unstable_modal" | "unstable_hideOnEsc" | "unstable_hideOnClickOutside" | "unstable_preventBodyScroll" | "unstable_initialFocusRef" | "unstable_finalFocusRef" | "unstable_autoFocusOnShow" | "unstable_autoFocusOnHide")[];
}
export declare const Dialog: <TT extends React.ElementType<any> = "div">({ as, ...props }: import("../__utils/types").PropsWithAs<unstable_DialogOptions, TT>, ref: React.Ref<any>) => JSX.Element;
